@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2009 Red Hat, Inc.
+ * Copyright (c) 2006-2012 Red Hat, Inc.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
@@ -44,7 +44,7 @@
 #else
 #define PROCESSOR_COUNT_MAX	384
 #define MESSAGE_SIZE_MAX	1024*1024 /* (1MB) */
-#define MESSAGE_QUEUE_MAX	MESSAGE_SIZE_MAX / totem_config->net_mtu
+#define MESSAGE_QUEUE_MAX	((4 * MESSAGE_SIZE_MAX) / totem_config->net_mtu)
 #endif /* HAVE_SMALL_MEMORY_FOOTPRINT */
 
 #define FRAME_SIZE_MAX		10000
@@ -52,10 +52,14 @@
 #define SEND_THREADS_MAX	16
 #define INTERFACE_MAX		2
 
-/*
+/**
  * Maximum number of continuous gather states
  */
 #define MAX_NO_CONT_GATHER	3
+/*
+ * Maximum number of continuous failures get from sendmsg call
+ */
+#define MAX_NO_CONT_SENDMSG_FAILURES	30
 
 struct totem_interface {
 	struct totem_ip_address bindnet;
@@ -65,23 +69,24 @@ struct totem_interface {
 	uint16_t ttl;
 	int member_count;
 	struct totem_ip_address member_list[PROCESSOR_COUNT_MAX];
-	
 };
 
 struct totem_logging_configuration {
 	void (*log_printf) (
-		unsigned int rec_ident,
+		int level,
+		int subsys,
 		const char *function_name,
 		const char *file_name,
 		int file_line,
 		const char *format,
-		...) __attribute__((format(printf, 5, 6)));
+		...) __attribute__((format(printf, 6, 7)));
 
 	int log_level_security;
 	int log_level_error;
 	int log_level_warning;
 	int log_level_notice;
 	int log_level_debug;
+	int log_level_trace;
 	int log_subsys_id;
 };
 
@@ -151,16 +156,6 @@ struct totem_config {
 
 	struct totem_logging_configuration totem_logging_configuration;
 
-	void (*log_rec) (
-		int subsysid,
-		const char *function_name,
-		const char *file_name,
-		int file_line,
-		unsigned int rec_ident,
-		...);
-
-	unsigned int secauth;
-
 	unsigned int net_mtu;
 
 	unsigned int threads;
@@ -177,15 +172,15 @@ struct totem_config {
 
 	unsigned int broadcast_use;
 
-	enum { TOTEM_CRYPTO_SOBER=0, TOTEM_CRYPTO_NSS } crypto_type;
-	enum { TOTEM_CRYPTO_ACCEPT_OLD=0, TOTEM_CRYPTO_ACCEPT_NEW } crypto_accept;
+	char *crypto_cipher_type;
 
-	int crypto_crypt_type;
-	int crypto_sign_type;
+	char *crypto_hash_type;
 
 	totem_transport_t transport_number;
 
 	unsigned int miss_count_const;
+
+	int ip_version;
 };
 
 #define TOTEM_CONFIGURATION_TYPE
@@ -212,7 +207,6 @@ struct memb_ring_id {
 } __attribute__((packed));
 
 typedef struct {
-	hdb_handle_t handle;
 	int is_dirty;
 	time_t last_updated;
 } totem_stats_header_t;
@@ -226,6 +220,8 @@ typedef struct {
 	totem_stats_header_t hdr;
 	totemnet_stats_t *net;
 	char *algo_name;
+	uint8_t *faulty;
+	uint32_t interface_count;
 } totemrrp_stats_t;
 
 
@@ -262,6 +258,7 @@ typedef struct {
 	uint64_t consensus_timeouts;
 	uint64_t rx_msg_dropped;
 	uint32_t continuous_gather;
+	uint32_t continuous_sendmsg_failures;
 
 	int earliest_token;
 	int latest_token;
