@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2009 Red Hat, Inc.
+ * Copyright (c) 2008-2012 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -42,11 +42,11 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <sys/uio.h>
 #include <errno.h>
 
+#include <qb/qbipcc.h>
 #include <corosync/corotypes.h>
-#include <corosync/coroipc_types.h>
-#include <corosync/coroipcc.h>
 #include <corosync/corodefs.h>
 #include <corosync/hdb.h>
 
@@ -56,20 +56,26 @@
 #include "util.h"
 
 struct quorum_inst {
-	hdb_handle_t handle;
+	qb_ipcc_connection_t *c;
 	int finalize;
 	const void *context;
 	quorum_callbacks_t callbacks;
 };
 
-DECLARE_HDB_DATABASE(quorum_handle_t_db,NULL);
+static void quorum_inst_free (void *inst);
+
+DECLARE_HDB_DATABASE(quorum_handle_t_db, quorum_inst_free);
 
 cs_error_t quorum_initialize (
 	quorum_handle_t *handle,
-	quorum_callbacks_t *callbacks)
+	quorum_callbacks_t *callbacks,
+	uint32_t *quorum_type)
 {
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
+	struct iovec iov;
+	struct qb_ipc_request_header req;
+	struct res_lib_quorum_gettype res_lib_quorum_gettype;
 
 	error = hdb_error_to_cs(hdb_handle_create (&quorum_handle_t_db, sizeof (struct quorum_inst), handle));
 	if (error != CS_OK) {
@@ -81,17 +87,35 @@ cs_error_t quorum_initialize (
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (
-		COROSYNC_SOCKET_NAME,
-		QUORUM_SERVICE,
-		IPC_REQUEST_SIZE,
-		IPC_RESPONSE_SIZE,
-		IPC_DISPATCH_SIZE,
-		&quorum_inst->handle);
+	error = CS_OK;
+	quorum_inst->finalize = 0;
+	quorum_inst->c = qb_ipcc_connect ("quorum", IPC_REQUEST_SIZE);
+	if (quorum_inst->c == NULL) {
+		error = qb_to_cs_error(-errno);
+		goto error_put_destroy;
+	}
+
+	req.size = sizeof (req);
+	req.id = MESSAGE_REQ_QUORUM_GETTYPE;
+
+	iov.iov_base = (char *)&req;
+	iov.iov_len = sizeof (req);
+
+	error = qb_to_cs_error(qb_ipcc_sendv_recv (
+		quorum_inst->c,
+		&iov,
+		1,
+		&res_lib_quorum_gettype,
+		sizeof (struct res_lib_quorum_gettype), -1));
+
 	if (error != CS_OK) {
 		goto error_put_destroy;
 	}
 
+	error = res_lib_quorum_gettype.header.error;
+
+	*quorum_type = res_lib_quorum_gettype.quorum_type;
+
 	if (callbacks)
 		memcpy(&quorum_inst->callbacks, callbacks, sizeof (*callbacks));
 	else
@@ -109,6 +133,12 @@ error_no_destroy:
 	return (error);
 }
 
+static void quorum_inst_free (void *inst)
+{
+	struct quorum_inst *quorum_inst = (struct quorum_inst *)inst;
+	qb_ipcc_disconnect(quorum_inst->c);
+}
+
 cs_error_t quorum_finalize (
 	quorum_handle_t handle)
 {
@@ -130,8 +160,6 @@ cs_error_t quorum_finalize (
 
 	quorum_inst->finalize = 1;
 
-	coroipcc_service_disconnect (quorum_inst->handle);
-
 	(void)hdb_handle_destroy (&quorum_handle_t_db, handle);
 
 	(void)hdb_handle_put (&quorum_handle_t_db, handle);
@@ -146,7 +174,7 @@ cs_error_t quorum_getquorate (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 	struct iovec iov;
-	coroipc_request_header_t req;
+	struct qb_ipc_request_header req;
 	struct res_lib_quorum_getquorate res_lib_quorum_getquorate;
 
 	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
@@ -160,12 +188,12 @@ cs_error_t quorum_getquorate (
 	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof (req);
 
-       error = coroipcc_msg_send_reply_receive (
-		quorum_inst->handle,
+	error = qb_to_cs_error(qb_ipcc_sendv_recv (
+		quorum_inst->c,
 		&iov,
 		1,
 		&res_lib_quorum_getquorate,
-		sizeof (struct res_lib_quorum_getquorate));
+		sizeof (struct res_lib_quorum_getquorate), CS_IPC_TIMEOUT_MS));
 
 	if (error != CS_OK) {
 		goto error_exit;
@@ -193,7 +221,7 @@ cs_error_t quorum_fd_get (
 		return (error);
 	}
 
-	error = coroipcc_fd_get (quorum_inst->handle, fd);
+	error = qb_to_cs_error(qb_ipcc_fd_get (quorum_inst->c, fd));
 
 	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 
@@ -248,7 +276,7 @@ cs_error_t quorum_trackstart (
 	struct quorum_inst *quorum_inst;
 	struct iovec iov;
 	struct req_lib_quorum_trackstart req_lib_quorum_trackstart;
-	coroipc_response_header_t res;
+	struct qb_ipc_response_header res;
 
 	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
@@ -262,12 +290,12 @@ cs_error_t quorum_trackstart (
 	iov.iov_base = (char *)&req_lib_quorum_trackstart;
 	iov.iov_len = sizeof (struct req_lib_quorum_trackstart);
 
-       error = coroipcc_msg_send_reply_receive (
-		quorum_inst->handle,
+       error = qb_to_cs_error(qb_ipcc_sendv_recv (
+		quorum_inst->c,
                 &iov,
                 1,
                 &res,
-                sizeof (res));
+                sizeof (res), CS_IPC_TIMEOUT_MS));
 
 	if (error != CS_OK) {
 		goto error_exit;
@@ -287,8 +315,8 @@ cs_error_t quorum_trackstop (
 	cs_error_t error;
 	struct quorum_inst *quorum_inst;
 	struct iovec iov;
-	coroipc_request_header_t req;
-	coroipc_response_header_t res;
+	struct qb_ipc_request_header req;
+	struct qb_ipc_response_header res;
 
 	error = hdb_error_to_cs(hdb_handle_get (&quorum_handle_t_db, handle, (void *)&quorum_inst));
 	if (error != CS_OK) {
@@ -301,12 +329,12 @@ cs_error_t quorum_trackstop (
 	iov.iov_base = (char *)&req;
 	iov.iov_len = sizeof (req);
 
-       error = coroipcc_msg_send_reply_receive (
-		quorum_inst->handle,
+       error = qb_to_cs_error(qb_ipcc_sendv_recv (
+		quorum_inst->c,
                 &iov,
                 1,
                 &res,
-                sizeof (res));
+                sizeof (res), CS_IPC_TIMEOUT_MS));
 
 	if (error != CS_OK) {
 		goto error_exit;
@@ -329,12 +357,14 @@ cs_error_t quorum_dispatch (
 	int cont = 1; /* always continue do loop except when set to 0 */
 	struct quorum_inst *quorum_inst;
 	quorum_callbacks_t callbacks;
-	coroipc_response_header_t *dispatch_data;
+	struct qb_ipc_response_header *dispatch_data;
+	char dispatch_buf[IPC_DISPATCH_SIZE];
 	struct res_lib_quorum_notification *res_lib_quorum_notification;
 
 	if (dispatch_types != CS_DISPATCH_ONE &&
 		dispatch_types != CS_DISPATCH_ALL &&
-		dispatch_types != CS_DISPATCH_BLOCKING) {
+		dispatch_types != CS_DISPATCH_BLOCKING &&
+		dispatch_types != CS_DISPATCH_ONE_NONBLOCKING) {
 
 		return (CS_ERR_INVALID_PARAM);
 	}
@@ -346,25 +376,33 @@ cs_error_t quorum_dispatch (
 	}
 
 	/*
-	 * Timeout instantly for CS_DISPATCH_ONE or SA_DISPATCH_ALL and
-	 * wait indefinately for CS_DISPATCH_BLOCKING
+	 * Timeout instantly for CS_DISPATCH_ONE_NONBLOCKING or CS_DISPATCH_ALL and
+	 * wait indefinately for CS_DISPATCH_ONE or CS_DISPATCH_BLOCKING
 	 */
-	if (dispatch_types == CS_DISPATCH_ALL) {
+	if (dispatch_types == CS_DISPATCH_ALL || dispatch_types == CS_DISPATCH_ONE_NONBLOCKING) {
 		timeout = 0;
 	}
 
+	dispatch_data = (struct qb_ipc_response_header *)dispatch_buf;
 	do {
-		error = coroipcc_dispatch_get (
-			quorum_inst->handle,
-			(void **)&dispatch_data,
-			timeout);
+		error = qb_to_cs_error (qb_ipcc_event_recv (
+			quorum_inst->c,
+			dispatch_buf,
+			IPC_DISPATCH_SIZE,
+			timeout));
 		if (error == CS_ERR_BAD_HANDLE) {
 			error = CS_OK;
 			goto error_put;
 		}
 		if (error == CS_ERR_TRY_AGAIN) {
+			if (dispatch_types == CS_DISPATCH_ONE_NONBLOCKING) {
+				/*
+				 * Don't mask error
+				 */
+				goto error_put;
+			}
 			error = CS_OK;
-			if (dispatch_types == CPG_DISPATCH_ALL) {
+			if (dispatch_types == CS_DISPATCH_ALL) {
 				break; /* exit do while cont is 1 loop */
 			} else {
 				continue; /* next poll */
@@ -399,28 +437,26 @@ cs_error_t quorum_dispatch (
 			break;
 
 		default:
-			error = coroipcc_dispatch_put (quorum_inst->handle);
-			if (error == CS_OK) {
-				error = CS_ERR_LIBRARY;
-			}
+			error = CS_ERR_LIBRARY;
 			goto error_put;
 			break;
 		}
-		error = coroipcc_dispatch_put (quorum_inst->handle);
-		if (error != CS_OK) {
+		if (quorum_inst->finalize) {
+			/*
+			 * If the finalize has been called then get out of the dispatch.
+			 */
+			error = CS_ERR_BAD_HANDLE;
 			goto error_put;
 		}
 
 		/*
 		 * Determine if more messages should be processed
 		 */
-		if (dispatch_types == CS_DISPATCH_ONE) {
+		if (dispatch_types == CS_DISPATCH_ONE || dispatch_types == CS_DISPATCH_ONE_NONBLOCKING) {
 			cont = 0;
 		}
 	} while (cont);
 
-	goto error_put;
-
 error_put:
 	(void)hdb_handle_put (&quorum_handle_t_db, handle);
 	return (error);
