@@ -84,7 +84,7 @@ static char * node_pid_format(unsigned int nodeid, unsigned int pid) {
 	static char buffer[100];
 	if (show_ip) {
 		struct in_addr saddr;
-#if __BYTE_ORDER == __BIG_ENDIAN
+#if __BYTE_ORDER == __LITTLE_ENDIAN
 		saddr.s_addr = swab32(nodeid);
 #else
 		saddr.s_addr = nodeid;
@@ -334,6 +334,11 @@ int main (int argc, char *argv[]) {
 	}
 
 	if (argc > optind) {
+		if (strlen(argv[optind]) >= CPG_MAX_NAME_LENGTH) {
+			fprintf(stderr, "Invalid name for cpg group\n");
+			return (1);
+		}
+
 		strcpy(group_name.value, argv[optind]);
 		group_name.length = strlen(argv[optind]);
 	}
